@@ -1,14 +1,22 @@
 # coding: utf-8
 
+# import native modules
 import gzip
 from datetime import datetime
+from time import gmtime, strftime
+
+# import installed modules
 from flask import current_app
 from flask.ext.script import Manager
 from sqlalchemy.exc import IntegrityError, InvalidRequestError
 from sqlalchemy.ext.serializer import dumps, loads
-from time import gmtime, strftime
 from unipath import Path
 
+# import alchemydumps helpers
+from helpers.autoclean import bw_lists
+from helpers.identification import get_bkp_dir, get_ids, get_list
+from helpers.sqlalchemy import get_sa_mapped_classes
+
 
 class _AlchemyDumpsConfig(object):
 
@@ -132,7 +140,7 @@ def remove(date_id):
     """Remove a series of backup files based on the date part of the files"""
 
     # check if date/id is valid
-    files = get_list()
+    files = get_list(date_id)
     if not date_id or date_id not in get_ids(files):
         print '==> Not a valid date id. Use the "history" command to list existing downloads'
         return
@@ -153,72 +161,48 @@ def remove(date_id):
             d.remove()
 
 
-def get_sa_mapped_classes(sqlalachemy_db=False):
+@AlchemyDumpsCommand.command
+def autoclean():
     """
-    :param sqlalachemy_db: (optional) SQLAlchemy database (if not passed, it is loaded from AlchemyDumps() instance)
-    :return: list of SQLALchemy mapped classes
+    Remove a series of backup files based on the following rules:
+    * It keeps all the backups from the last 7 days
+    * It keeps one (the most recent one) backup from each week of the last month
+    * It keeps one (the most recent one) backup from each month of the last year
+    * It keeps one (the most recent one) backup from each year of the remaining years
     """
-    if not sqlalachemy_db:
-        sqlalachemy_db = current_app.extensions['alchemydumps'].db
-    do_not_backup = []
-    sa_models = list()
-    for m in sqlalachemy_db.Model.__subclasses__():
-        if m not in do_not_backup:
-            sa_models.append(m)
-    return sa_models
 
+    # check if there are backups
+    files = get_list()
+    date_ids = get_ids(files)
+    if not len(files):
+        print '==> No backups found.'
+        return
 
-def get_bkp_dir():
-    """
-    :return: unipath object of the dir where backup files are saved
-    """
-    bkp_dir = current_app.extensions['alchemydumps'].basedir.child('alchemydumps')
-    if not bkp_dir.exists():
-        bkp_dir.mkdir()
-    return bkp_dir
+    # get black and white list
+    lists = bw_lists(date_ids)
+    if not len(lists['blacklist']):
+        print '==> No backup to be deleted.'
+        return
 
+    # print the list of files to be kept/deleted
+    blacklist = list()
+    for l in ['whitelist', 'blacklist']:
+        msg = 'kept' if l == 'whitelist' else 'deleted'
+        print '\n==> {} backups will be {}:'.format(len(lists[l]), msg)
+        for date_id in lists[l]:
+            date_transf = datetime.strptime(date_id, '%Y%m%d%H%M%S')
+            date_format = date_transf.strftime('%b %d, %Y at %H:%M:%S')
+            print '\n    ID: {} (from {})'.format(date_id, date_format)
+            for f in get_list(date_id, files):
+                print '    {}'.format(f)
+                if l == 'blacklist':
+                    blacklist.append(f)
 
-def get_id(filepath):
-    """
-    :param filepath: unipath object of a file generated by AlchemyDumps
-    :return: the backup numeric id
-    """
-    filename = filepath.stem
-    parts = filename.split('-')
-    try:
-        return parts[2]
-    except IndexError:
-        return False
+    # confirm
+    confirmation = raw_input('\n==> Press "Y" to confirm, or anything else to abort.')
+    if confirmation.lower() == 'y':
 
-
-def get_list(date_id=False, files=False):
-    """
-    :param date_id: (optional) Backup file numeric id (if False, list everything)
-    :param files: (optional) If you have already called get_list(), pass the file list to improve performance
-    :return: The list of backup files from that id
-    """
-    if files:
-        if date_id:
-            output = [f for f in files if date_id in f.stem]
-        else:
-            output = files
-    else:
-        bkp_dir = get_bkp_dir()
-        pattern = '*{}*'.format(date_id) if date_id else None
-        output = [f.absolute() for f in bkp_dir.listdir(pattern)]
-    return output
-
-
-def get_ids(files=False):
-    """
-    :param files: (optional) If you have already called get_list(), pass the file list to improve performance
-    :return: List all valid IDs from the backup folder
-    """
-    if not files:
-        files = get_list()
-    file_ids = list()
-    for f in files:
-        file_id = get_id(f)
-        if file_id and not file_ids.count(file_id):
-            file_ids.append(file_id)
-    return file_ids
\ No newline at end of file
+        # delete
+        for filepath in blacklist:
+            print '    {} deleted.'.format(filepath.absolute())
+            filepath.remove()
